#!/usr/bin/env python3
"""
Windows Battery Monitor (System Management Tool)

Parameters:
    --status: Show current battery status
    --power-usage: Show power usage statistics
    --power-plans: List available power plans
    --set-plan GUID: Set active power plan
    --monitor [INTERVAL] [THRESHOLD]: Monitor battery with alerts
    --format FORMAT: Output format (table/json)

Requires: psutil, win32api, tabulate packages
Warning: Some features may require specific hardware support
"""

import sys
import json
import time
import logging
import subprocess
from typing import List, Dict, Optional
from datetime import datetime

import psutil
import win32com.client
from tabulate import tabulate

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s'
)
logger = logging.getLogger(__name__)

class BatteryMonitor:
    """Monitors and manages battery status on Windows laptops."""
    
    def __init__(self):
        """Initialize the BatteryMonitor."""
        self.wmi = win32com.client.GetObject("winmgmts:")
        
    def get_battery_status(self) -> Dict:
        """
        Get current battery status.
        
        Returns:
            Dict: Battery status information
        """
        battery = psutil.sensors_battery()
        if not battery:
            logger.error("No battery found")
            return {
                "error": "No battery found",
                "timestamp": datetime.now().isoformat()
            }
            
        # Get additional battery info from WMI
        battery_info = self.wmi.ExecQuery("SELECT * FROM Win32_Battery")[0]
        
        return {
            "percent": battery.percent,
            "power_plugged": battery.power_plugged,
            "time_left": battery.secsleft if battery.secsleft >= 0 else "Unknown",
            "status": self._get_battery_status(battery_info.BatteryStatus),
            "voltage": battery_info.DesignVoltage,
            "name": battery_info.Name,
            "chemistry": battery_info.Chemistry,
            "design_capacity": battery_info.DesignCapacity,
            "full_charge_capacity": battery_info.FullChargeCapacity,
            "health": self._calculate_battery_health(
                battery_info.DesignCapacity,
                battery_info.FullChargeCapacity
            ),
            "timestamp": datetime.now().isoformat()
        }
    
    def get_power_usage(self) -> Dict:
        """
        Get current power usage statistics.
        
        Returns:
            Dict: Power usage information
        """
        try:
            # Get power usage data using powercfg
            cmd = ["powercfg", "/srumutil", "/output", "power_log.csv", "/csv"]
            subprocess.run(cmd, capture_output=True, text=True)
            
            # Process the CSV file (implementation depends on format)
            power_data = self._process_power_log()
            
            return {
                "current_usage": power_data.get("current_usage", "Unknown"),
                "average_usage": power_data.get("average_usage", "Unknown"),
                "peak_usage": power_data.get("peak_usage", "Unknown"),
                "timestamp": datetime.now().isoformat()
            }
            
        except Exception as e:
            logger.error(f"Error getting power usage: {e}")
            return {
                "error": str(e),
                "timestamp": datetime.now().isoformat()
            }
    
    def get_power_plans(self) -> List[Dict]:
        """
        Get available power plans.
        
        Returns:
            List[Dict]: List of power plans
        """
        plans = []
        
        try:
            # Get power plans using powercfg
            cmd = ["powercfg", "/list"]
            result = subprocess.run(cmd, capture_output=True, text=True)
            
            for line in result.stdout.splitlines():
                if "GUID" in line:
                    guid = line.split()[3]
                    name = line.split("(")[1].split(")")[0]
                    active = "*" in line
                    
                    plans.append({
                        "guid": guid,
                        "name": name,
                        "active": active
                    })
                    
        except Exception as e:
            logger.error(f"Error getting power plans: {e}")
            
        return plans
    
    def set_power_plan(self, guid: str) -> bool:
        """
        Set the active power plan.
        
        Args:
            guid: GUID of the power plan to activate
            
        Returns:
            bool: True if successful, False otherwise
        """
        try:
            cmd = ["powercfg", "/setactive", guid]
            result = subprocess.run(cmd, capture_output=True, text=True)
            
            if result.returncode == 0:
                logger.info(f"Successfully set power plan: {guid}")
                return True
            else:
                logger.error(f"Error setting power plan: {result.stderr}")
                return False
                
        except Exception as e:
            logger.error(f"Error setting power plan: {e}")
            return False
    
    def monitor_battery(self, interval: int = 60, threshold: int = 20) -> None:
        """
        Continuously monitor battery status and send alerts.
        
        Args:
            interval: Monitoring interval in seconds
            threshold: Battery percentage threshold for alerts
        """
        logger.info(f"Starting battery monitoring (threshold: {threshold}%)")
        
        try:
            while True:
                status = self.get_battery_status()
                
                if status.get("error"):
                    logger.error(status["error"])
                    break
                    
                if status["percent"] <= threshold and not status["power_plugged"]:
                    self._send_low_battery_alert(status["percent"])
                    
                logger.info(f"Battery: {status['percent']}% " + 
                          f"({'Plugged In' if status['power_plugged'] else 'On Battery'})")
                
                time.sleep(interval)
                
        except KeyboardInterrupt:
            logger.info("Battery monitoring stopped")
    
    def _get_battery_status(self, status_code: int) -> str:
        """
        Convert battery status code to string.
        
        Args:
            status_code: Battery status code
            
        Returns:
            str: Status description
        """
        status_map = {
            1: "Discharging",
            2: "AC Power",
            3: "Fully Charged",
            4: "Low",
            5: "Critical",
            6: "Charging",
            7: "Charging and High",
            8: "Charging and Low",
            9: "Charging and Critical",
            10: "Undefined",
            11: "Partially Charged"
        }
        return status_map.get(status_code, "Unknown")
    
    def _calculate_battery_health(self, design_capacity: int, 
                                full_charge_capacity: int) -> float:
        """
        Calculate battery health percentage.
        
        Args:
            design_capacity: Original design capacity
            full_charge_capacity: Current full charge capacity
            
        Returns:
            float: Battery health percentage
        """
        if design_capacity and full_charge_capacity:
            return round((full_charge_capacity / design_capacity) * 100, 1)
        return 0.0
    
    def _process_power_log(self) -> Dict:
        """
        Process power usage log file.
        
        Returns:
            Dict: Processed power usage data
        """
        # This is a placeholder implementation
        # Actual implementation would parse the CSV file
        return {
            "current_usage": "Unknown",
            "average_usage": "Unknown",
            "peak_usage": "Unknown"
        }
    
    def _send_low_battery_alert(self, percent: float) -> None:
        """
        Send a low battery alert.
        
        Args:
            percent: Current battery percentage
        """
        try:
            # Use Windows notification
            cmd = [
                "powershell",
                f'[System.Windows.Forms.MessageBox]::Show("Battery level is at {percent}%", "Low Battery Warning", "OK", "Warning")'
            ]
            subprocess.run(cmd)
        except Exception as e:
            logger.error(f"Error sending battery alert: {e}")

def main():
    """Main function to handle command-line interface."""
    import argparse
    
    parser = argparse.ArgumentParser(description="Windows Battery Monitor")
    parser.add_argument("--status", action="store_true",
                      help="Show current battery status")
    parser.add_argument("--power-usage", action="store_true",
                      help="Show power usage statistics")
    parser.add_argument("--list-plans", action="store_true",
                      help="List available power plans")
    parser.add_argument("--set-plan",
                      help="Set power plan by GUID")
    parser.add_argument("--monitor", action="store_true",
                      help="Start continuous monitoring")
    parser.add_argument("--interval", type=int, default=60,
                      help="Monitoring interval in seconds")
    parser.add_argument("--threshold", type=int, default=20,
                      help="Low battery threshold percentage")
    parser.add_argument("--format", choices=["table", "json"], default="table",
                      help="Output format (table/json)")
    
    args = parser.parse_args()
    
    battery_monitor = BatteryMonitor()
    
    try:
        if args.status:
            status = battery_monitor.get_battery_status()
            if args.format == "json":
                print(json.dumps(status, indent=2))
            else:
                if "error" in status:
                    print(f"Error: {status['error']}")
                else:
                    data = [
                        ["Percentage", f"{status['percent']}%"],
                        ["Power Source", "AC Power" if status['power_plugged'] else "Battery"],
                        ["Time Left", f"{status['time_left']} seconds" if isinstance(status['time_left'], int) else status['time_left']],
                        ["Status", status['status']],
                        ["Health", f"{status['health']}%"],
                        ["Battery Name", status['name']],
                        ["Chemistry", status['chemistry']],
                        ["Design Capacity", status['design_capacity']],
                        ["Full Charge Capacity", status['full_charge_capacity']]
                    ]
                    print(tabulate(data, headers=["Property", "Value"], 
                                 tablefmt="grid"))
                
        elif args.power_usage:
            usage = battery_monitor.get_power_usage()
            if args.format == "json":
                print(json.dumps(usage, indent=2))
            else:
                if "error" in usage:
                    print(f"Error: {usage['error']}")
                else:
                    data = [
                        ["Current Usage", usage['current_usage']],
                        ["Average Usage", usage['average_usage']],
                        ["Peak Usage", usage['peak_usage']]
                    ]
                    print(tabulate(data, headers=["Metric", "Value"], 
                                 tablefmt="grid"))
                
        elif args.list_plans:
            plans = battery_monitor.get_power_plans()
            if args.format == "json":
                print(json.dumps(plans, indent=2))
            else:
                headers = ["GUID", "Name", "Active"]
                table_data = [[p["guid"], p["name"], "Yes" if p["active"] else "No"]
                            for p in plans]
                print(tabulate(table_data, headers=headers, tablefmt="grid"))
                
        elif args.set_plan:
            if battery_monitor.set_power_plan(args.set_plan):
                print(f"Successfully set power plan: {args.set_plan}")
            else:
                print(f"Failed to set power plan: {args.set_plan}")
                
        elif args.monitor:
            battery_monitor.monitor_battery(args.interval, args.threshold)
            
        else:
            parser.print_help()
            
    except Exception as e:
        logger.error(f"An error occurred: {e}")
        sys.exit(1)

if __name__ == "__main__":
    main()
